/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import javax.xml.namespace.QName;

import org.bluestemsoftware.open.eoa.commons.util.DOMUtils;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationInfaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationOutfaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationOutputElement;
import org.w3c.dom.Element;

public class InterfaceOperationElementImpl extends AbstractWSDLElement implements InterfaceOperationElement {

    private static final long serialVersionUID = 1L;

    public InterfaceOperationElementImpl(WSDL20DocumentImpl owner) {
        super(owner, NAME.getNamespaceURI(), NAME.getPrefix() + ":" + NAME.getLocalPart());
    }

    public InterfaceOperationElementImpl() {
        super();
    }

    public void setName(String name) {
        setAttribute("name", name);
    }

    public String getName() {
        return getAttribute("name").equals("") ? null : getAttribute("name");
    }

    public void setPattern(String value) {
        setAttribute("pattern", value);
    }

    public String getPattern() {
        return getAttribute("pattern").equals("") ? null : getAttribute("pattern");
    }

    public void setStyles(Set<String> styles) {
        if (styles == null) {
            return;
        }
        StringBuilder sb = new StringBuilder("");
        String delim = "";
        for (String style : styles) {
            sb.append(delim);
            sb.append(style);
            delim = " ";
        }
        setAttribute("style", sb.toString());
    }

    public Set<String> getStyles() {
        Set<String> styles = new HashSet<String>();
        StringTokenizer st = new StringTokenizer(getAttribute("style"));
        while (st.hasMoreTokens()) {
            styles.add(st.nextToken());
        }
        return styles;
    }
    
    public boolean isSafe() {
        return getAttributeNS(ExtendedDocument.WSDL_EXT_NS, "safe").equals("")
        ? false
        : Boolean.parseBoolean(getAttributeNS(ExtendedDocument.WSDL_EXT_NS, "safe"));
    }

    public void setIsSafe(boolean isSafe) {
        setAttributeNS(ExtendedDocument.WSDL_EXT_NS, ExtendedDocument.WSDL_EXT_NS_PREFIX
                + ":safe", Boolean.toString(isSafe));
    }

    public InterfaceOperationInputElement addInputElement(InterfaceOperationInputElement inputElement) {
        super.insertBefore(inputElement, null);
        return inputElement;
    }

    public List<InterfaceOperationInputElement> getInputElements() {
        List<Element> temp = DOMUtils.getChildElements(this, new QName(InterfaceOperationInputElement.NAME
                .getNamespaceURI(), InterfaceOperationInputElement.NAME.getLocalPart()));
        List<InterfaceOperationInputElement> answer = new ArrayList<InterfaceOperationInputElement>();
        for (Element element : temp) {
            answer.add((InterfaceOperationInputElement)element);
        }
        return answer;
    }

    public InterfaceOperationOutputElement addOutputElement(InterfaceOperationOutputElement outputElement) {
        super.insertBefore(outputElement, null);
        return outputElement;
    }

    public List<InterfaceOperationOutputElement> getOutputElements() {
        List<Element> temp = DOMUtils.getChildElements(this, new QName(InterfaceOperationOutputElement.NAME
                .getNamespaceURI(), InterfaceOperationOutputElement.NAME.getLocalPart()));
        List<InterfaceOperationOutputElement> answer = new ArrayList<InterfaceOperationOutputElement>();
        for (Element element : temp) {
            answer.add((InterfaceOperationOutputElement)element);
        }
        return answer;
    }

    public InterfaceOperationInfaultElement addInfaultElement(InterfaceOperationInfaultElement infaultElement) {
        super.insertBefore(infaultElement, null);
        return infaultElement;
    }

    public List<InterfaceOperationInfaultElement> getInfaultElements() {
        List<Element> temp = DOMUtils.getChildElements(this, new QName(InterfaceOperationInfaultElement.NAME
                .getNamespaceURI(), InterfaceOperationInfaultElement.NAME.getLocalPart()));
        List<InterfaceOperationInfaultElement> answer = new ArrayList<InterfaceOperationInfaultElement>();
        for (Element element : temp) {
            answer.add((InterfaceOperationInfaultElement)element);
        }
        return answer;
    }

    public InterfaceOperationOutfaultElement addOutfaultElement(InterfaceOperationOutfaultElement outfaultElement) {
        super.insertBefore(outfaultElement, null);
        return outfaultElement;
    }

    public List<InterfaceOperationOutfaultElement> getOutfaultElements() {
        List<Element> temp = DOMUtils.getChildElements(this, new QName(InterfaceOperationOutfaultElement.NAME
                .getNamespaceURI(), InterfaceOperationOutfaultElement.NAME.getLocalPart()));
        List<InterfaceOperationOutfaultElement> answer = new ArrayList<InterfaceOperationOutfaultElement>();
        for (Element element : temp) {
            answer.add((InterfaceOperationOutfaultElement)element);
        }
        return answer;
    }
    
}
