/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.xerces.dom.DocumentImpl;
import org.bluestemsoftware.open.eoa.commons.util.QNameUtils;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.ext.ActorElementImpl;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.ext.ApplicationElementImpl;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.ext.EngineElementImpl;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.ext.HTTPHeaderElementImpl;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.ext.RoleElementImpl;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.ext.SOAPHeaderElementImpl;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.util.Constants;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.util.DOMSerializer;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.util.DocumentContext;
import org.bluestemsoftware.specification.eoa.ext.Extension;
import org.bluestemsoftware.specification.eoa.ext.ExtensionFactory;
import org.bluestemsoftware.specification.eoa.ext.schema.xs.xs10.XMLSchema;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.WSDLTypesElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingFaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingOperationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingOperationInfaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingOperationOutfaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingOperationOutputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.DescriptionElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ImportElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.IncludeElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceFaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationInfaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationOutfaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceOperationOutputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ServiceElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ServiceEndpointElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.TypesElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.ActorElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.ApplicationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.EngineElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.HTTPBindingElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.HTTPBindingFaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.HTTPBindingOperationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.HTTPBindingOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.HTTPBindingOperationOutputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.HTTPHeaderElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.RoleElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.SOAPBindingElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.SOAPBindingFaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.SOAPBindingOperationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.SOAPBindingOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.SOAPBindingOperationOutputElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.SOAPHeaderElement;
import org.bluestemsoftware.specification.eoa.system.SystemContext;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class WSDL20DocumentImpl extends DocumentImpl implements ExtendedDocument, ExtendedDocument.Provider {

    private static final long serialVersionUID = 1L;
    private DocumentContext documentContext;
    private int prefixCounter = 1;
    
    

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.RootExtension#getExtensionType()
     */
    public final String getExtensionType() {
        return WSDL20Document.TYPE;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.Extension#getExtensionFactory()
     */
    public final ExtensionFactory getExtensionFactory() {
        return SystemContext.getContext().getSystem().getIDLFactory(WSDL20Document.TYPE);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.Extension#getExtensionProvider()
     */
    public org.bluestemsoftware.specification.eoa.ext.Extension.Provider getExtensionProvider() {
        return this;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.Extension$Provider#spi_setConsumer(org.bluestemsoftware.specification.eoa.ext.Extension)
     */
    public void spi_setConsumer(Extension consumer) {
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.WSDLDocument$Provider#addNamespace(java.lang.String)
     */
    public synchronized String addNamespace(String ns) {
        DescriptionElement descriptionElement = getDescriptionElement();
        if (descriptionElement == null) {
            return null;
        }
        String prefix = lookupPrefix(ns);
        if (prefix == null) {
            String tns = descriptionElement.getTargetNamespace();
            if (tns != null && tns.equals(ns)) {
                prefix = "tns";
            } else {
                String counterAsString = String.valueOf(prefixCounter++);
                if (counterAsString.length() == 1) {
                    counterAsString = "0" + counterAsString;
                }
                prefix = "ns" + counterAsString;
            }
            descriptionElement.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:" + prefix, ns);
        }
        return prefix;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.WSDLDocument$Provider#getTargetNamespace()
     */
    public String getTargetNamespace() {
        return getDescriptionElement().getTargetNamespace();
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.WSDLDocument$Provider#getTypesElement()
     */
    public WSDLTypesElement getTypesElement() {
        return getDescriptionElement().getTypesElement();
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#getDescriptionElement()
     */
    public DescriptionElement getDescriptionElement() {
        return (DescriptionElement)getDocumentElement();
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createDescriptionElement()
     */
    public DescriptionElement createDescriptionElement() {
        return new DescriptionElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#setDescriptionElement(org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.DescriptionElement)
     */
    public void setDescriptionElement(DescriptionElement descriptionElement) {
        appendChild(descriptionElement);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createImportElement()
     */
    public ImportElement createImportElement() {
        return new ImportElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createIncludeElement()
     */
    public IncludeElement createIncludeElement() {
        return new IncludeElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createTypesElement()
     */
    public TypesElement createTypesElement() {
        return new TypesElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createInterfaceElement()
     */
    public InterfaceElement createInterfaceElement() {
        return new InterfaceElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createInterfaceFaultElement()
     */
    public InterfaceFaultElement createInterfaceFaultElement() {
        return new InterfaceFaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createInterfaceOperationElement()
     */
    public InterfaceOperationElement createInterfaceOperationElement() {
        return new InterfaceOperationElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createInterfaceOperationInputElement()
     */
    public InterfaceOperationInputElement createInterfaceOperationInputElement() {
        return new InterfaceOperationInputElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createInterfaceOperationOutputElement()
     */
    public InterfaceOperationOutputElement createInterfaceOperationOutputElement() {
        return new InterfaceOperationOutputElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createInterfaceOperationInfaultElement()
     */
    public InterfaceOperationInfaultElement createInterfaceOperationInfaultElement() {
        return new InterfaceOperationInfaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createInterfaceOperationOutfaultElement()
     */
    public InterfaceOperationOutfaultElement createInterfaceOperationOutfaultElement() {
        return new InterfaceOperationOutfaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createBindingElement()
     */
    public BindingElement createBindingElement() {
        return new BindingElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createBindingFaultElement()
     */
    public BindingFaultElement createBindingFaultElement() {
        return new BindingFaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createBindingOperationElement()
     */
    public BindingOperationElement createBindingOperationElement() {
        return new BindingOperationElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createBindingOperationInputElement()
     */
    public BindingOperationInputElement createBindingOperationInputElement() {
        return new BindingOperationInputElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createBindingOperationOutputElement()
     */
    public BindingOperationOutputElement createBindingOperationOutputElement() {
        return new BindingOperationOutputElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createBindingOperationInfaultElement()
     */
    public BindingOperationInfaultElement createBindingOperationInfaultElement() {
        return new BindingOperationInfaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createBindingOperationOutfaultElement()
     */
    public BindingOperationOutfaultElement createBindingOperationOutfaultElement() {
        return new BindingOperationOutfaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createServiceElement()
     */
    public ServiceElement createServiceElement() {
        return new ServiceElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createServiceEndpointElement()
     */
    public ServiceEndpointElement createServiceEndpointElement() {
        return new ServiceEndpointElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createApplicationElement()
     */
    public ApplicationElement createApplicationElement() {
        return new ApplicationElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createEngineElement()
     */
    public EngineElement createEngineElement() {
        return new EngineElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createRoleElement()
     */
    public RoleElement createRoleElement() {
        return new RoleElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document$Provider#createActorElement()
     */
    public ActorElement createActorElement() {
        return new ActorElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument$Provider#createHTTPHeaderElement()
     */
    public HTTPHeaderElement createHTTPHeaderElement() {
        return new HTTPHeaderElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument$Provider#createSOAPBindingElement()
     */
    public SOAPBindingElement createSOAPBindingElement() {
        return new BindingElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument$Provider#createSOAPBindingFaultElement()
     */
    public SOAPBindingFaultElement createSOAPBindingFaultElement() {
        return new BindingFaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument$Provider#createSOAPBindingOperationElement()
     */
    public SOAPBindingOperationElement createSOAPBindingOperationElement() {
        return new BindingOperationElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument$Provider#createSOAPBindingOperationInputElement()
     */
    public SOAPBindingOperationInputElement createSOAPBindingOperationInputElement() {
        return new BindingOperationInputElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument$Provider#createSOAPBindingOperationOutputElement()
     */
    public SOAPBindingOperationOutputElement createSOAPBindingOperationOutputElement() {
        return new BindingOperationOutputElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument$Provider#createSOAPHeaderElement()
     */
    public SOAPHeaderElement createSOAPHeaderElement() {
        return new SOAPHeaderElementImpl(this);
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument#createHTTPBindingElement()
     */
    public HTTPBindingElement createHTTPBindingElement() {
        return new BindingElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument#createHTTPBindingFaultElement()
     */
    public HTTPBindingFaultElement createHTTPBindingFaultElement() {
        return new BindingFaultElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument#createHTTPBindingOperationElement()
     */
    public HTTPBindingOperationElement createHTTPBindingOperationElement() {
        return new BindingOperationElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument#createHTTPBindingOperationInputElement()
     */
    public HTTPBindingOperationInputElement createHTTPBindingOperationInputElement() {
        return new BindingOperationInputElementImpl(this);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ExtendedDocument#createHTTPBindingOperationOutputElement()
     */
    public HTTPBindingOperationOutputElement createHTTPBindingOperationOutputElement() {
        return new BindingOperationOutputElementImpl(this);
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.idl.IDLDocument#getGrammarDocuments()
     */
    public Set<Document> getGrammarDocuments() {

        Set<Document> schemaDocuments = new HashSet<Document>();

        WSDLTypesElement typesElement = getTypesElement();

        if (typesElement == null) {
            return schemaDocuments;
        }

        DocumentBuilder db;
        try {
            db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        } catch (ParserConfigurationException pe) {
            throw new IllegalStateException("Error retrieving grammar documents. " + pe);
        }
        
        for (Element schemaElement : getChildElements(typesElement, new QName(XMLSchema.TYPE, "*"))) {

            Document document = db.newDocument();

            if (schemaElement.getLocalName().equals("import")) {

                document.appendChild(document.importNode(schemaElement, true));

            } else {

                String tns = schemaElement.getAttribute("targetNamespace");

                if (tns.equals("")) {

                    throw new IllegalStateException("Schema targetNamespace undefined on definition "
                            + " inlined within wsdl 2.0 document at URI "
                            + getDocumentURI()
                            + ".");

                } else {

                    document.appendChild(document.importNode(schemaElement, true));

                }

            }
            
            schemaDocuments.add(document);

        }

        return schemaDocuments;

    }


    /**
     * Overrides ancestor such that we can intercept invocations by DOMParser and return a wsdl
     * element if qname matches an expected name. Note that in some cases, qname of some
     * elements are the same and we must therefore examine context within which element is
     * being created, i.e. whose the parent, to return the correct element.
     */
    @Override
    public Element createElementNS(String namespaceURI, String qualifiedName, String localpart) throws DOMException {
        QName elementName = new QName(namespaceURI, localpart);
        if (elementName.equals(DescriptionElement.NAME)) {
            return new DescriptionElementImpl(this);
        }
        if (elementName.equals(ImportElement.NAME)) {
            return new ImportElementImpl(this);
        }
        if (elementName.equals(IncludeElement.NAME)) {
            return new IncludeElementImpl(this);
        }
        if (elementName.equals(TypesElement.NAME)) {
            return new TypesElementImpl(this);
        }
        if (elementName.equals(InterfaceElement.NAME)) {
            return new InterfaceElementImpl(this);
        }
        if (elementName.equals(InterfaceFaultElement.NAME)) {
            QName parentName = documentContext.getParentName();
            if (parentName != null && parentName.equals(InterfaceElement.NAME)) {
                return new InterfaceFaultElementImpl(this);
            }
        }
        if (elementName.equals(InterfaceOperationElement.NAME)) {
            QName parentName = documentContext.getParentName();
            if (parentName != null && parentName.equals(InterfaceElement.NAME)) {
                return new InterfaceOperationElementImpl(this);
            }
        }
        if (elementName.equals(InterfaceOperationInputElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(InterfaceOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(InterfaceElement.NAME)) {
                        return new InterfaceOperationInputElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(InterfaceOperationOutputElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(InterfaceOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(InterfaceElement.NAME)) {
                        return new InterfaceOperationOutputElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(InterfaceOperationInfaultElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(InterfaceOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(InterfaceElement.NAME)) {
                        return new InterfaceOperationInfaultElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(InterfaceOperationOutfaultElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(InterfaceOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(InterfaceElement.NAME)) {
                        return new InterfaceOperationOutfaultElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(BindingElement.NAME)) {
            return new BindingElementImpl(this);
        }
        if (elementName.equals(SOAPBindingElement.NAME)) {
            return new BindingElementImpl(this);
        }
        if (elementName.equals(BindingFaultElement.NAME)) {
            QName parentName = documentContext.getParentName();
            if (parentName != null && parentName.equals(BindingElement.NAME)) {
                return new BindingFaultElementImpl(this);
            }
        }
        if (elementName.equals(BindingOperationElement.NAME)) {
            QName parentName = documentContext.getParentName();
            if (parentName != null && parentName.equals(BindingElement.NAME)) {
                return new BindingOperationElementImpl(this);
            }
        }
        if (elementName.equals(BindingOperationInputElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(BindingOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(BindingElement.NAME)) {
                        return new BindingOperationInputElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(BindingOperationOutputElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(BindingOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(BindingElement.NAME)) {
                        return new BindingOperationOutputElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(BindingOperationInfaultElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(BindingOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(BindingElement.NAME)) {
                        return new BindingOperationInfaultElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(BindingOperationOutfaultElement.NAME)) {
            Element parent = documentContext.getParent();
            if (parent != null) {
                QName parentName = new QName(parent.getNamespaceURI(), parent.getLocalName());
                if (parentName.equals(BindingOperationElement.NAME)) {
                    Element grandParent = (Element)parent.getParentNode();
                    QName grandParentName = new QName(grandParent.getNamespaceURI(), grandParent.getLocalName());
                    if (grandParentName.equals(BindingElement.NAME)) {
                        return new BindingOperationOutfaultElementImpl(this);
                    }
                }
            }
        }
        if (elementName.equals(ServiceElement.NAME)) {
            return new ServiceElementImpl(this);
        }
        if (elementName.equals(ServiceEndpointElement.NAME)) {
            return new ServiceEndpointElementImpl(this);
        }
        if (elementName.equals(ApplicationElement.NAME)) {
            return new ApplicationElementImpl(this);
        }
        if (elementName.equals(RoleElement.NAME)) {
            return new RoleElementImpl(this);
        }
        if (elementName.equals(EngineElement.NAME)) {
            QName parentName = documentContext.getParentName();
            if (parentName != null && parentName.equals(DescriptionElement.NAME)) {
                return new EngineElementImpl(this);
            }
        }
        if (elementName.equals(ActorElement.NAME)) {
            QName parentName = documentContext.getParentName();
            if (parentName != null && parentName.equals(EngineElement.NAME)) {
                return new ActorElementImpl(this);
            }
        }
        if (elementName.equals(SOAPHeaderElement.NAME)) {
            return new SOAPHeaderElementImpl(this);
        }
        if (elementName.equals(HTTPHeaderElement.NAME)) {
            return new HTTPHeaderElementImpl(this);
        }
        return super.createElementNS(namespaceURI, qualifiedName, localpart);
    }

    @Override
    public Element createElementNS(String namespaceURI, String qualifiedName) throws DOMException {
        String localPart = QNameUtils.getLocalPart(qualifiedName);
        return createElementNS(namespaceURI, qualifiedName, localPart);
    }

    public void setDocumentContext(DocumentContext documentContext) {
        this.documentContext = documentContext;
    }

    public DocumentContext getDocumentContext() {
        return documentContext;
    }

    @Override
    public String toString() {
        StringWriter writer = new StringWriter();
        try {
            DOMSerializer.serializeNode(this, writer, Constants.UTF_8, true);
        } catch (Exception ex) {
            return super.toString();
        }
        return writer.toString();
    }
    
    private List<Element> getChildElements(Element parent, QName childName) {
        List<Element> answer = new ArrayList<Element>();
        NodeList nodeList = parent.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element element = (Element)nodeList.item(i);
            if (childName.getNamespaceURI().equals(XMLConstants.NULL_NS_URI)) {
                if (element.getNamespaceURI() != null) {
                    continue;
                }
            } else {
                if (element.getNamespaceURI() == null) {
                    continue;
                }
                if (!childName.getNamespaceURI().equals("*")
                        && !element.getNamespaceURI().equals(childName.getNamespaceURI())) {
                    continue;
                }
            }
            if (!childName.getLocalPart().equals("*") && !element.getLocalName().equals(childName.getLocalPart())) {
                continue;
            }
            answer.add(element);
        }
        return answer;
    }

}
